@@ -768,9 +768,9 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 	     log(L_ERR, "Acct: MySQL connection died");
 	} else {
 
-                       /* If backup file exists we know the database was down */
-                       if(stat(MYSQLBACKUP, &backup) == 0) {
-                            if(backup.st_size > 0) {
+ 		/* If backup file exists we know the database was down */
+		if(stat(MYSQLBACKUP, &backup) == 0) {
+			if(backup.st_size > 0) {
 
                               /* We'll fork a child to load records in the backup file */
 
@@ -809,16 +809,13 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
                                    exit(0);
                                 }
                             }
-                      }
+		}
 
-	}
-
-        if (MySock != NULL) {
-           ret = mysql_save_acct(sqlrecord);
-           if (!mysql_keepopen) {
-             log(L_DBG, "Acct: Closing connection to MySQL server on %s", mysql_server);
-  	     mysql_close(MySock);
-           }
+		ret = mysql_save_acct(sqlrecord);
+		if (!mysql_keepopen) {
+			mysql_close(MyAcctSock);
+			MyAcctSock = NULL;
+		}
         } 
 #endif
 
