@@ -111,7 +111,6 @@ static int check_expiration(VALUE_PAIR *check_item, char *umsg, char **user_msg)
  *	Check the users password against the standard UNIX
  *	password table.
  */
-#ifndef USEMYSQL
 static int unix_pass(char *name, char *passwd)
 {
 	struct passwd	*pwd;
@@ -214,7 +213,6 @@ static int unix_pass(char *name, char *passwd)
 
 	return 0;
 }
-#endif
 
 
 /*
@@ -246,9 +244,7 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 	int		i;
 	int		strip_username;
 	int		result;
-#ifndef USEMYSQL
 	char		*ptr;
-#endif
 	/*
 	 *	cjd 19980706 --
 	 *	pampair contains the pair of PAM_AUTH_ATTR
@@ -350,15 +346,6 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 		strcpy(userpass, string);
 	}
 
-/* Since we read the users stuff into the structures it acts as if AUTHTYPE was local */
-#ifdef USEMYSQL
-
-	strncpy(dbpass, password_pair->strvalue, 127);
-	if (password_pair == NULL || strcmp(password_pair->strvalue, string)!=0)
-		result = -1;
-
-#else
-
 
 #if 0 /* DEBUG */
 	printf("auth_type=%d, string=%s, namepair=%s, password_pair=%s\n",
@@ -414,6 +401,11 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 				/*
 				 *	Plain text password.
 				 */
+#ifdef USEMYSQL
+				strncpy(dbpass, password_pair->strvalue, 127);
+#else
+				*dbpass = '\0';
+#endif
 				if (password_pair == NULL ||
 				    strcmp(password_pair->strvalue, string)!=0)
 					result = -1;
@@ -469,7 +461,6 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 			result = -1;
 			break;
 	}
-#endif
 
 	if (result < 0)
 		*user_msg = NULL;
@@ -618,6 +609,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	/*
 	 *	Get the user from the database
 	 */
+        if (MyAuthSock != NULL )
+ 	 	DEBUG("MyAuthSock is NOT null before user_find");
 	if (user_find(namepair->strvalue, authreq->request,
 	   &user_check, &user_reply) != 0) {
 		log(L_AUTH, "Invalid user: [%s] (%s)",
@@ -725,15 +718,16 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		unsigned long	diff = 0;
 
 		allowed = check_item->lvalue * 3600;
+		mysql_checksocket("Auth");
 		sprintf(querystr, "SELECT SUM(AcctSessionTime) FROM radacct WHERE UserName = '%s' and AcctStartTime >= CONCAT(YEAR(NOW()),'-',MONTH(NOW()),'-01 00:00:00') AND AcctStartTime <= CONCAT(YEAR(NOW()),'-',MONTH(NOW())+1,'-01 00:00:00')", namepair->strvalue);
 		if (sqltrace)
 			DEBUG(querystr);
-		mysql_query(MySock, querystr);
-		if (!(sqlresult = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+		mysql_query(MyAuthSock, querystr);
+		if (!(sqlresult = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
 			log(L_ERR,"MYSQL Error: Cannot get result");
-			log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-			mysql_close(MySock);
-			MySock = NULL;
+			log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+			mysql_close(MyAuthSock);
+			MyAuthSock = NULL;
 		} else {
 			row = mysql_fetch_row(sqlresult);
 			used = atol(row[0]);
@@ -799,9 +793,9 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 			}
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
 				user_reply, user_msg, activefd);
-		log(L_ERR, "Multiple logins: [%s] (from nas %s) max. %d%s",
+		log(L_ERR, "Multiple logins: [%s] (%s) max. %d%s",
 				namepair->strvalue,
-				nas_name2(authreq),
+				auth_name(authreq, 1),
 				check_item->lvalue,
 				r == 2 ? " [MPP attempt]" : "");
 			result = -1;
@@ -971,7 +965,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 
 	rad_send_reply(PW_AUTHENTICATION_ACK, authreq,
 			user_reply, user_msg, activefd);
-	if (log_auth_pass) {
+	if (log_auth) {
 #if 1 /* Hide the password for `miquels' :) */
 		if (strcmp(namepair->strvalue, "miquels") == 0)
 			strcpy(userpass, "guess");
@@ -995,6 +989,10 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	authfree(authreq);
 	pairfree(user_check);
 	pairfree(user_reply);
+	if (!mysql_keepopen) {
+		mysql_close(MyAuthSock);
+		MyAuthSock = NULL;
+	}
 
 	return 0;
 }
