@@ -44,7 +44,8 @@ int mysql_start ()
         int     line_no;
         char    buffer[256];
         char    sqlfile[256];
-	MySock = NULL;
+	MyAuthSock = NULL;
+	MyAcctSock = NULL;
        
        strcpy(mysql_server,"localhost");
        strcpy(mysql_login,"");
@@ -200,24 +201,22 @@ int mysql_start ()
        }
        fclose(sqlfd);
        
-       log(L_INFO,"MYSQL_init: using: %s,%s,%s,%s",
+       log(L_INFO,"MYSQL: Attempting to connect to %s:%s as %s",
        mysql_server,
-       mysql_login,
        mysql_db,
-       mysql_acct_table);
+       mysql_login);
 
        if (mysql_keepopen) {
            /* Connect to the database server */
-           if (!(MySock = mysql_connect(&MyConn, mysql_server, mysql_login, mysql_password))) {
-                log(L_ERR, "Init: Couldn't connect to MySQL server on %s as %s", mysql_server, mysql_login);
-                MySock = NULL;
-           } else {
-                /* Select the database */
-                if (mysql_select_db(MySock, mysql_db) == -1) {
-                        log(L_ERR, "Init: Couldn't select MySQL database %s", mysql_db);
-                        mysql_close(MySock);
-                        MySock = NULL;
-                }
+           mysql_init(&MyAuthConn);
+           if (!(MyAuthSock = mysql_real_connect(&MyAuthConn, mysql_server, mysql_login, mysql_password, mysql_db, 0, NULL, 0))) {
+                log(L_ERR, "Init: Couldn't connect authentication socket to MySQL server on %s as %s", mysql_server, mysql_login);
+                MyAuthSock = NULL;
+           }
+           mysql_init(&MyAcctConn);
+           if (!(MyAcctSock = mysql_real_connect(&MyAcctConn, mysql_server, mysql_login, mysql_password, mysql_db, 0, NULL, 0))) {
+                log(L_ERR, "Init: Couldn't connect accounting socket to MySQL server on %s as %s", mysql_server, mysql_login);
+                MyAcctSock = NULL;
            }
        }
            
@@ -256,14 +255,15 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
      if (sqlrecord.AcctStatusTypeId == PW_STATUS_ACCOUNTING_ON || sqlrecord.AcctStatusTypeId == PW_STATUS_ACCOUNTING_OFF) {
         log(L_INFO, "Portmaster %s rebooted at %s", sqlrecord.NASIPAddress, sqlrecord.AcctTimeStamp);
     
-   	if (MySock != NULL) {
+   	if (MyAcctSock != NULL) {
   
           /* The Terminal server informed us that it was rebooted
           * STOP all records from this NAS */
 
              sprintf(querystr, "UPDATE %s SET AcctStopTime='%s', AcctSessionTime=unix_timestamp('%s') - unix_timestamp(AcctStartTime), AcctTerminateCause='%s' WHERE AcctSessionTime=0 AND AcctStopTime=0 AND NASIPAddress= '%s' AND AcctStartTime <= '%s'", mysql_acct_table, sqlrecord.AcctTimeStamp, sqlrecord.AcctTimeStamp, sqlrecord.AcctTerminateCause, sqlrecord.NASIPAddress, sqlrecord.AcctTimeStamp);
 
-              mysql_query(MySock, querystr);
+       	      if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
+	          log(L_ERR, "Acct: Couldn't update SQL accounting after NAS reboot - %s", mysql_error(MyAcctSock));
 
               if (mysqlfile) {
                   fputs(querystr, mysqlfile);
@@ -295,7 +295,7 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
         return 0;
      } 
 
-     if (MySock != NULL) {
+     if (MyAcctSock != NULL) {
 
           /* Got start record */
           if(sqlrecord.AcctStatusTypeId == PW_STATUS_START) {
@@ -310,10 +310,10 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
              );
  	     if (sqltrace)
 	        DEBUG(querystr);
-       	     if (mysql_query(MySock, (const char *) querystr) < 0)
-	        log(L_ERR, "Acct: Couldn't update SQL accounting START record - %s", mysql_error(MySock));
+       	     if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
+	        log(L_ERR, "Acct: Couldn't update SQL accounting START record - %s", mysql_error(MyAcctSock));
 
-             num = mysql_affected_rows(MySock);
+             num = mysql_affected_rows(MyAcctSock);
              if (num == 0) {
 
                 /* Insert new record with blank stop time until stop record is got */
@@ -337,28 +337,26 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
 
  		if (sqltrace)
 		   DEBUG(querystr);
-       	        if (mysql_query(MySock, (const char *) querystr) < 0)
-	   	  log(L_ERR, "Acct: Couldn't insert SQL accounting START record - %s", mysql_error(MySock));
+       	        if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
+	   	  log(L_ERR, "Acct: Couldn't insert SQL accounting START record - %s", mysql_error(MyAcctSock));
              }
 
            /* Got stop record */
            } else {
 
-             
-
              sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND UserName='%s'", mysql_acct_table, sqlrecord.AcctSessionId, sqlrecord.NASIPAddress, sqlrecord.UserName);
  	      if (sqltrace)
   	        DEBUG(querystr);
-              mysql_query(MySock, querystr);
-              if (!(result = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+              mysql_query(MyAcctSock, querystr);
+              if (!(result = mysql_store_result(MyAcctSock)) && mysql_num_fields(MyAcctSock)) {
                    log(L_ERR,"MYSQL Error: Cannot get result");
-                   log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-                    mysql_close(MySock);
-                    MySock = NULL;
+                   log(L_ERR,"MYSQL error: %s",mysql_error(MyAcctSock));
+                    mysql_close(MyAcctSock);
+                    MyAcctSock = NULL;
               } else {
                     num = mysql_num_rows(result);
+	       	    mysql_free_result(result);
               }
-	      mysql_free_result(result);
 
              if (num > 0) {
 
@@ -378,10 +376,10 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
 
  	        if (sqltrace)
   	          DEBUG(querystr);
-       	        if (mysql_query(MySock, (const char *) querystr) < 0)
-	           log(L_ERR, "Acct: Couldn't update SQL accounting STOP record - %s", mysql_error(MySock));
+       	        if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
+	           log(L_ERR, "Acct: Couldn't update SQL accounting STOP record - %s", mysql_error(MyAcctSock));
 
-             } else {
+             } else if (num == 0) {
 
             
                 /* Insert record with no start time until matching start record comes */
@@ -409,8 +407,8 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
 
  	        if (sqltrace)
   	           DEBUG(querystr);
-       	        if (mysql_query(MySock, (const char *) querystr) < 0)
-		   log(L_ERR, "Acct: Couldn't insert SQL accounting STOP record - %s", mysql_error(MySock));
+       	        if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
+		   log(L_ERR, "Acct: Couldn't insert SQL accounting STOP record - %s", mysql_error(MyAcctSock));
              }
 
           }
@@ -599,26 +597,35 @@ int mysql_userparse(VALUE_PAIR **first_pair, MYSQL_ROW row, int mode) {
  *************************************************************************/
 int mysql_checksocket(const char *facility) {
 
-    if (MySock == NULL) {
-           if (mysql_keepopen)
-              log(L_ERR, "%s: Keepopen set but had to reconnect to MySQL", facility);
-           /* Connect to the database server */
-           if (!(MySock = mysql_connect(&MyConn, mysql_server, mysql_login, mysql_password))) {
-                log(L_ERR, "%s: Couldn't connect to MySQL server on %s as %s", facility, mysql_server, mysql_login);
-                MySock = NULL;
-		return 0;
-           } else {
-                /* Select the database */
-                if (mysql_select_db(MySock, mysql_db) == -1) {
-                        log(L_ERR, "%s: Couldn't select MySQL database %s", facility, mysql_db);
-                        mysql_close(MySock);
-                        MySock = NULL;
-			return 0;
-                }
-           }
-    }
-    return 1;
+	if ((strncmp(facility, "Auth", 4) == 0)) {
+		if (MyAuthSock == NULL) {
+			if (mysql_keepopen)
+				log(L_ERR, "%s: Keepopen set but had to reconnect to MySQL", facility);
+			/* Connect to the database server */
+			mysql_init(&MyAuthConn);
+			if (!(MyAuthSock = mysql_real_connect(&MyAuthConn, mysql_server, mysql_login, mysql_password, mysql_db, 0, NULL, 0))) {
+				log(L_ERR, "Auth: Couldn't connect authentication socket to MySQL server on %s as %s", mysql_server, mysql_login);
+				MyAuthSock = NULL;
+				return 0;
+			}
+		}
 
+	} else {
+		if (MyAcctSock == NULL) {
+			if (mysql_keepopen)
+				log(L_ERR, "%s: Keepopen set but had to reconnect to MySQL", facility);
+			/* Connect to the database server */
+			mysql_init(&MyAcctConn);
+			if (!(MyAcctSock = mysql_real_connect(&MyAcctConn, mysql_server, mysql_login, mysql_password, mysql_db, 0, NULL, 0))) {
+				log(L_ERR, "Acct: Couldn't connect accounting socket to MySQL server on %s as %s", mysql_server, mysql_login);
+				MyAcctSock = NULL;
+				return 0;
+			}
+		}
+
+	}
+
+	return 1;
 
 }
 
@@ -643,12 +650,13 @@ int mysql_getvpdata(char *table, VALUE_PAIR **vp, char *user, int mode) {
 		sprintf(querystr, "SELECT %s.* FROM %s, %s WHERE %s.UserName = '%s' AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, mysql_usergroup_table, mysql_usergroup_table, user, mysql_usergroup_table, table, table);
 	if (sqltrace)
 		DEBUG(querystr);
-	mysql_query(MySock, querystr);
-	if (!(result = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+        mysql_checksocket("Auth");
+	mysql_query(MyAuthSock, querystr);
+	if (!(result = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
    		log(L_ERR,"MYSQL Error: Cannot get result");
-   		log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-   		mysql_close(MySock);
-  		MySock = NULL;
+   		log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+   		mysql_close(MyAuthSock);
+  		MyAuthSock = NULL;
 	} else {
 		rows = mysql_num_rows(result);
 		while ((row = mysql_fetch_row(result))) {
@@ -782,12 +790,12 @@ int mysql_check_multi(char *name, VALUE_PAIR *request, int maxsimul) {
 	sprintf(querystr, "SELECT COUNT(*) FROM %s WHERE UserName = '%s' AND AcctStopTime = 0", mysql_acct_table, name);
 	if (sqltrace)
 		DEBUG(querystr);
-	mysql_query(MySock, querystr);
-	if (!(result = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+	mysql_query(MyAuthSock, querystr);
+	if (!(result = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
    		log(L_ERR,"MYSQL Error: Cannot get result");
-   		log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-   		mysql_close(MySock);
-  		MySock = NULL;
+   		log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+   		mysql_close(MyAuthSock);
+  		MyAuthSock = NULL;
 	} else {
 		row = mysql_fetch_row(result);
 		count = atoi(row[0]);
@@ -807,12 +815,12 @@ int mysql_check_multi(char *name, VALUE_PAIR *request, int maxsimul) {
 	sprintf(querystr, "SELECT * FROM %s WHERE UserName = '%s' AND AcctStopTime = 0", mysql_acct_table, name);
 	if (sqltrace)
 		DEBUG(querystr);
-	mysql_query(MySock, querystr);
-	if (!(result = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+	mysql_query(MyAuthSock, querystr);
+	if (!(result = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
    		log(L_ERR,"MYSQL Error: Cannot get result");
-   		log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-   		mysql_close(MySock);
-  		MySock = NULL;
+   		log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+   		mysql_close(MyAuthSock);
+  		MyAuthSock = NULL;
 	} else {
 		while ((row = mysql_fetch_row(result))) {
 			if (mysql_check_ts(row) == 1) {
@@ -829,7 +837,7 @@ int mysql_check_multi(char *name, VALUE_PAIR *request, int maxsimul) {
 				sprintf(querystr, "DELETE FROM %s WHERE RadAcctId = '%s'", mysql_acct_table, row[0]);
 				if (sqltrace)
 					DEBUG(querystr);
-				mysql_query(MySock, querystr);
+				mysql_query(MyAuthSock, querystr);
 				
 			}
 		}
@@ -854,17 +862,17 @@ int mysql_read_naslist(char *nastable)
         nas_free(naslist);
         naslist = NULL;
 
-	if (!mysql_checksocket("Init"))
+	if (!mysql_checksocket("Auth"))
 		return -1;
 	sprintf(querystr, "SELECT * FROM %s", nastable);
 	if (sqltrace)
 		DEBUG(querystr);
-	mysql_query(MySock, querystr);
-	if (!(result = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+	mysql_query(MyAuthSock, querystr);
+	if (!(result = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
    		log(L_ERR,"MYSQL Error: Cannot get result");
-   		log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-   		mysql_close(MySock);
-  		MySock = NULL;
+   		log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+   		mysql_close(MyAuthSock);
+  		MyAuthSock = NULL;
 	} else {
 		while ((row = mysql_fetch_row(result))) {
 			if ((c = malloc(sizeof(NAS))) == NULL) {
@@ -880,6 +888,7 @@ int mysql_read_naslist(char *nastable)
         	        c->next = naslist;
                 	naslist = c;
         	}
+		mysql_free_result(result);
 	}
 
         return 0;
@@ -899,17 +908,17 @@ int mysql_read_clients(char *nastable)
         clients_free(clients);
         clients = NULL;
 
-	if (!mysql_checksocket("Init"))
+	if (!mysql_checksocket("Auth"))
 		return -1;
 	sprintf(querystr, "SELECT * FROM %s", nastable);
 	if (sqltrace)
 		DEBUG(querystr);
-	mysql_query(MySock, querystr);
-	if (!(result = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+	mysql_query(MyAuthSock, querystr);
+	if (!(result = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
    		log(L_ERR,"MYSQL Error: Cannot get result");
-   		log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-   		mysql_close(MySock);
-  		MySock = NULL;
+   		log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+   		mysql_close(MyAuthSock);
+  		MyAuthSock = NULL;
 	} else {
 		while ((row = mysql_fetch_row(result))) {
 			if ((c = malloc(sizeof(CLIENT))) == NULL) {
@@ -925,6 +934,7 @@ int mysql_read_clients(char *nastable)
         	        c->next = clients;
                 	clients = c;
         	}
+		mysql_free_result(result);
 	}
 
         return 0;
@@ -955,7 +965,7 @@ int mysql_dict_init(void) {
 #endif
 
 
-	if (!mysql_checksocket("Init"))
+	if (!mysql_checksocket("Auth"))
 		return -1;
 
 	dict_free();
@@ -964,12 +974,12 @@ int mysql_dict_init(void) {
 	sprintf(querystr, "SELECT * FROM %s", mysql_dict_table);
 	if (sqltrace)
 		DEBUG(querystr);
-	mysql_query(MySock, querystr);
-	if (!(result = mysql_store_result(MySock)) && mysql_num_fields(MySock)) {
+	mysql_query(MyAuthSock, querystr);
+	if (!(result = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
    		log(L_ERR,"MYSQL Error: Cannot get result");
-   		log(L_ERR,"MYSQL error: %s",mysql_error(MySock));
-   		mysql_close(MySock);
-  		MySock = NULL;
+   		log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+   		mysql_close(MyAuthSock);
+  		MyAuthSock = NULL;
 	} else {
 		while ((row = mysql_fetch_row(result))) {
 			line_no = atoi(row[0]);
@@ -1009,6 +1019,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 			"dict_init: Invalid name length on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 	
@@ -1016,6 +1027,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 				"dict_init: Invalid value on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 				if (row[3][0] != '0')
@@ -1039,6 +1051,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 				"dict_init: Invalid type on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 	
@@ -1050,6 +1063,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 				"dict_init: unknown vendor %s on line %d of dictionary",
 					row[5], line_no);
+					mysql_free_result(result);
 					return -1;
 				}
 
@@ -1057,6 +1071,7 @@ int mysql_dict_init(void) {
 				if((attr = (DICT_ATTR *)malloc(sizeof(DICT_ATTR))) ==
 						(DICT_ATTR *)NULL) {
 					log(L_ERR|L_CONS, "dict_init: out of memory");
+					mysql_free_result(result);
 					return(-1);
 				}
 				strcpy(attr->name, row[2]);
@@ -1084,6 +1099,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 			"dict_init: Invalid attribute length on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 	
@@ -1091,6 +1107,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 			"dict_init: Invalid name length on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 	
@@ -1098,6 +1115,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 				"dict_init: Invalid value on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 				value = atoi(row[4]);
@@ -1106,6 +1124,7 @@ int mysql_dict_init(void) {
 				if((dval = (DICT_VALUE *)malloc(sizeof(DICT_VALUE))) ==
 						(DICT_VALUE *)NULL) {
 					log(L_ERR|L_CONS, "dict_init: out of memory");
+					mysql_free_result(result);
 					return(-1);
 				}
 				strcpy(dval->attrname, row[2]);
@@ -1126,6 +1145,7 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 			"dict_init: Invalid attribute length on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 	
@@ -1133,19 +1153,23 @@ int mysql_dict_init(void) {
 					log(L_ERR|L_CONS,
 				"dict_init: Invalid value on line %d of dictionary",
 						line_no);
+					mysql_free_result(result);
 					return(-1);
 				}
 				value = atoi(row[3]);
 	
 				/* Create a new VENDOR entry for the list */
-				if (addvendor(row[2], value) < 0)
+				if (addvendor(row[2], value) < 0) {
+					mysql_free_result(result);
 					return -1;
+				}
 #ifdef ATTRIB_NMC
 				if (value == VENDORPEC_USR)
 					vendor_usr_seen = 1;
 #endif
 			}
 		}
+		mysql_free_result(result);
 	}
 		return(0);
 }
