@@ -153,6 +153,10 @@ static void reread_config(int reload)
 		}
 		exit(1);
 	}
+	if (!mysql_keepopen) {
+		mysql_close(MyAuthSock);
+		MyAuthSock = NULL;
+	}
 }
 
 
@@ -722,6 +726,7 @@ static void rad_spawn_child(AUTH_REQ *authreq, int activefd, FUNP fun)
 		request_list_busy = 0;
 		signal(SIGCHLD, SIG_DFL);
 		(*fun)(authreq, activefd);
+
 		exit(0);
 	}
 
