@@ -345,8 +345,10 @@ char mysql_nas_table[MAX_TABLE_LEN];
 char mysql_dict_table[MAX_TABLE_LEN];
 int  mysql_keepopen;
 int  sqltrace; 
-MYSQL *MySock;
-MYSQL MyConn;
+MYSQL *MyAuthSock;
+MYSQL *MyAcctSock;
+MYSQL MyAuthConn;
+MYSQL MyAcctConn;
 int		mysql_start();
 int		mysql_save_acct(MYSQLREC sqlrecord);
 int		mysql_userparse(VALUE_PAIR **first_pair, MYSQL_ROW row, int mode);
